import { objectHasKey } from '@/utils/helper';
import { getBlockConnection } from '../helper';

async function interactionHandler(block, prevBlockData) {
  const nextBlockId = getBlockConnection(block);

  try {
    const refData = {
      prevBlockData,
      dataColumns: this.data,
      loopData: this.loopData,
      globalData: this.globalData,
      activeTabUrl: this.activeTabUrl,
    };
    const data = await this._sendMessageToTab(
      { ...block, refData },
      {
        frameId: this.frameId || 0,
      }
    );

    if (block.name === 'link')
      await new Promise((resolve) => setTimeout(resolve, 5000));

    if (data?.isError) {
      const error = new Error(data.message);
      error.nextBlockId = nextBlockId;

      throw error;
    }

    if (objectHasKey(block.data, 'dataColumn')) {
      if (!block.data.saveData)
        return {
          data,
          nextBlockId,
        };

      const currentColumnType =
        this.columns[block.data.dataColumn]?.type || 'any';

      if (Array.isArray(data) && currentColumnType !== 'array') {
        data.forEach((item) => {
          this.addData(block.data.dataColumn, item);
        });
      } else {
        this.addData(block.data.dataColumn, data);
      }
    } else if (block.name === 'javascript-code') {
      const arrData = Array.isArray(data) ? data : [data];

      this.addData(arrData);
    }

    return {
      data,
      nextBlockId,
    };
  } catch (error) {
    error.nextBlockId = nextBlockId;

    throw error;
  }
}

export default interactionHandler;
